-- 本代码作者是Alexliyu, 依据游戏源代码以及开发文档进行编写, 并遵守与继承游戏本身的协议.如果需要二次修改,提取或者分发请注明出处
DefineClass.BaseAttacker = {
	__parents = {
		"AttackRover"
	},
	need_spawn = false,
	allow_roam = false,
	use_random_pos = true,
	can_repair = true,
	pin_on_start = true,
	display_name = T{"超级战车"},
	description = T{"火星大魔王派来的超级战车，目的就是赶你出火星"},
	reclaimed_description = T{"火星大魔王赠送的超级战车，不但可以打怪还可以打天打地打自己"},
	battery_max = 200000,
	battery_hourly_drain_rate = 1000,
	accumulate_dust = false,
	dust_devil_malfunction_chance = 0,
	pfclass = 1,
	collision_radius = 540,
	direction_arrow_scale = 260,
	roam_target_radius = 750 * guim,
	attack_range = 500 * guim,
	attack_target_labels = {
		"OutsideBuildingsTargets",
		"Rover",
		"Drone",
		"Colonist",
	},
	reload_time = const.HourDuration / 10,
	reload_expiration_time = false,
	reclaimed = true,
	max_health = 100,
	rocket_damage = 5,
	current_health = false,
	attacks_remaining = false,
	last_attacker = false,
	meteor = false,
	track_thread = false,
	intercept_distance_percent = 80,
	intercept_reload_percent = 10,
	is_repair_request_initialized = false,
	-- entity = "Female",
	-- entity = "DroneLOD",
	-- default_skins = {
	-- "Drone_Trailblazer",
	-- "DroneLOD"
	-- },
	ip_template = "ipAttackRover",
	palettes = {"AttackRoverRed"},
	fx_actor_class = "AttackRover"
}

local UnbuildableZ = buildUnbuildableZ()
local IsPlayablePoint = function(pt)
  local x, y = pt:xy()
  local q, r = WorldToHex(x, y)
  return pt:InBox2D(g_MapArea) and GetBuildableZ(q, r) ~= UnbuildableZ and terrain.IsPassable(pt)
end

function BaseAttacker:DoAttack(target)
	self:FireRocket({target = target})
	self:SetCommand("Reload")
end

function BaseAttacker:Attack(num_attacks, attacker)
  local target
  if not attacker then
    if num_attacks and num_attacks ~= 0 then
      self.attacks_remaining = Max(0, num_attacks) - 1
    else
      self.attacks_remaining = nil
      self:SetCommand("Roam")
    end
  end
  self.attack_look_for_target = true
  while true do
    target = self:PickTarget()
    if not target then
	  -- 敌人自动漫游， 我方如果开启允许漫游则开始漫游
      if not self.reclaimed or self.allow_roam then
        self:RoamTick()
      else
        Sleep(1000)
        self:SetCommand("Idle")
      end
    else
      break
    end
  end
  self.attack_look_for_target = nil
  -- mod_print("原始目标%s, 距离%d", target.name, self:GetVisualDist2D(target))
  self:MoveInRange(target)
  local is_building = IsKindOf(target, "Building")
  if not IsValid(target) or is_building and target.destroyed or not is_building and target:IsDead() then
    Sleep(1000)
    return
  end
  -- 这里应该设置循环， 循环到达目的地
  if self:GetVisualDist2D(target) > self.attack_range then
	Sleep(1000)
	-- mod_print("当前目标%s, 距离%d", target.name, self:GetVisualDist2D(target))
    -- return self:SetCommand("Idle")
	-- CommandObject.SetCommand(self, "Idle")
	return self:Idle()
  end
  self:DoAttack(target)
end

function BaseAttacker:MoveInRange(target)
  if self:GetVisualDist2D(target) <= self.attack_range then
    return
  end
  local is_building = IsKindOf(target, "Building")
  if is_building and target.destroyed or not is_building and target:IsDead() then
    return
  end
  -- mod_print("%s向目标%s开始移动, 距离%d", self.name, target.name, self.attack_range)
  if self.reclaimed then
    -- mod_print(self:GetRadius()*20)
	-- self:Goto(target, 24000, 0)
	-- self:Goto(target, self.attack_range, 0)
	self:Goto(target, Min(24000, self.attack_range), 0)
  else
	self:Goto(target, self.attack_range, 0)
  end
end

function BaseAttacker:PickTarget()
  local city = self.city or UICity
  local target, target_dist
  if self.reclaimed then
    if IsValid(self.last_attacker) then
		if IsKindOf(self.last_attacker, "Building") and not self.last_attacker.destroyed and not self.last_attacker:IsMalfunctioned() then
			return self.last_attacker
		end
		if not IsKindOf(self.last_attacker, "Building") and not (self.last_attacker:IsDead() or self.last_attacker:IsMalfunctioned()) then
			return self.last_attacker
		end
	end
    local hostile = self.city.labels.HostileAttackers or empty_table
    for _, obj in ipairs(hostile) do
      if IsValid(obj) and self:GetVisualDist2D(obj) <= self.attack_range then
        return obj
      end
    end
  else
    if IsValid(self.last_attacker) then
      if IsKindOf(self.last_attacker, "Building") and not self.last_attacker.destroyed and not self.last_attacker:IsMalfunctioned() then
        return self.last_attacker
      end
      local hostile = self.city.labels.HostileAttackers or empty_table
      if not table.find(hostile, self.last_attacker) and not self.last_attacker:IsDead() then
        return self.last_attacker
      end
    end
    local pos = self:GetPos()
    for _, label in ipairs(self.attack_target_labels) do
      local list = city.labels[label] or empty_table
      local count = #list
      local offset = self:Random(count)
      for i = 1, #list do
        local obj = list[(offset + i) % count + 1]
        local is_building = obj:IsKindOf("Building")
        if is_building and not obj.destroyed and not obj.parent_dome or not is_building and not obj:IsDead() then
          target = obj
          break
        end
      end
    end
  end
  return target
end

function BaseAttacker:Repair()
	self.battery_current = self.battery_max
	local city = self.city or UICity
	self:DisconnectFromCommandCenters()
	self.current_health = self.max_health
	self.malfunction_idle_state = nil
	self:SetState("idle")
	self.is_repair_request_initialized = false
	self.reclaimed = true
	self.palettes = {
	  "AttackRoverRed"
	}
	SetPaletteFromClassMember(self)
	city:RemoveFromLabel("HostileAttackers", self)
	city:AddToLabel("Rover", self)
	CommandObject.SetCommand(self, "Idle")
    Msg("AttackRoverRepaired", self)
    ObjModified(self)
end

function BaseAttacker:GetRefundResources()
  return {
    {
      resource = "Metals",
      amount = 100 * const.ResourceScale
    },
    {
      resource = "Electronics",
      amount = 100 * const.ResourceScale
    },
	{
      resource = "PreciousMetals",
      amount = 100 * const.ResourceScale
    },
  }
end

function BaseAttacker:CanInteractWithObject(obj, interaction_mode)
  if self.command == "Dead" then
    return false
  end
  if not self.reclaimed then
    return false, false, true
  end
  if interaction_mode == false or interaction_mode == "attack" then
    if IsKindOf(obj, "Building") or IsKindOf(obj, "BaseRover") or IsKindOf(obj, "Drone") or IsKindOf(obj, "Colonist") then
	    if obj.command ~= "Dead" and obj.command ~= "Malfunction" then
		   return true
		end
		return false
    else
		return false
    end
  end
  if (self.interaction_mode == false or self.interaction_mode == "default" or self.interaction_mode == "move") and IsKindOf(obj, "Tunnel") and obj.working then
    return true
  end
  return false
end

function BaseAttacker:InteractWithObject(obj, interaction_mode)
  if not IsValid(obj) then
    return false
  end
  if self.command == "Dead" then
    return false
  end
  if interaction_mode == false or interaction_mode == "recharge" then
    if IsKindOf(obj, "ElectricityGridElement") and not IsKindOf(obj, "ConstructionSite") then
      self:SetCommand("RechargeFromGrid", obj)
      SetUnitControlInteractionMode(self, false)
    elseif obj ~= self and IsKindOf(obj, "BaseRover") and obj.command ~= "Dead" then
      self:SetCommand("EqualizePowerWithOtherRover", obj)
      SetUnitControlInteractionMode(self, false)
    end
  end
 
  if interaction_mode == false or interaction_mode == "attack" then
    if IsKindOf(obj, "Building") or IsKindOf(obj, "BaseRover") or IsKindOf(obj, "Drone") or IsKindOf(obj, "Colonist") then
		if obj.command ~= "Dead" and obj.command ~= "Malfunction" then
			self.last_attacker = obj
			-- self:SetCommand("Attack", -1)
			-- self:Goto(obj:GetPos())
			-- CommandObject.SetCommand(self, "Attack", -1)
			CommandObject.SetCommand(self, "Idle")
			-- self:SetCommand("Goto", obj:GetPos())
			SetUnitControlInteractionMode(self, false)
		end
    end
  end
  if (self.interaction_mode == false or self.interaction_mode == "default" or self.interaction_mode == "move") and IsKindOf(obj, "Tunnel") and obj.working then
    self:SetCommand("UseTunnel", obj)
    SetUnitControlInteractionMode(self, false)
  end
end

function BaseAttacker:CanHit(meteor)
  if not self.reclaimed or self.command ~= "Idle" then
    return false
  end
  local hostile = self.city.labels.HostileAttackers or empty_table
  for _, obj in ipairs(hostile) do
    if self:GetVisualDist2D(obj) <= self.attack_range then
      return false
    end
  end
  if not IsValid(meteor) or self:IsExecuting() or IsValid(meteor.marked) and meteor.marked ~= self or IsValidThread(self.track_thread) or HexAxialDistance(self, meteor.dest) > self.protect_range then
    return false
  end
  return true
end

function BaseAttacker:NoBattery()
  local city = self.city or UICity
  city:RemoveFromLabel("HostileAttackers", self)
  Msg("AttackRoverMalfunctioned", self)
  BaseRover.NoBattery(self)
end

function BaseAttacker:Malfunction()
  local city = self.city or UICity
  city:RemoveFromLabel("HostileAttackers", self)
  self.malfunction_idle_state = self.current_health == 0 and "destroyed" or "idle"
  Msg("AttackRoverMalfunctioned", self)
  BaseRover.Malfunction(self, "suppress")
end

-- 无人机好像不能修理自己，应该是实体类的问题，暂时只能战车自己修理
function BaseAttacker:Dead()
  if not self.reclaimed then
    Msg("HostileOnDead", self)
    Msg("AttackRoverDead", self)
    self:SetCommand("Malfunction")
  end
  local city = self.city or UICity
  city:RemoveFromLabel("HostileAttackers", self)
  BaseRover.Dead(self)
  ObjModified(self)
end

function BaseAttacker:ShowWorkRadius(show, reason)
  if show then
    if next(WorkRadiusShownForRoverReasons) == nil then
      WorkRadiusShownForRover = self
      PlayFX("ShowWorkRadius", "start", self)
    end
    WorkRadiusShownForRoverReasons[reason] = true
  elseif WorkRadiusShownForRover == self then
    WorkRadiusShownForRoverReasons[reason] = nil
    if next(WorkRadiusShownForRoverReasons) == nil then
      PlayFX("ShowWorkRadius", "end", self)
      WorkRadiusShownForRover = false
    end
  end
end

function BaseAttacker:RocketDamage(shooter)
  if self:IsDead() or self:IsMalfunctioned() then
    self.city:RemoveFromLabel("HostileAttackers", self)
  end
  if self:IsDead() then
    return
  end
  -- 使用对方的攻击值作为伤害
  self.current_health = Max(0, self.current_health - (shooter.rocket_damage or self.rocket_damage))
  if self.current_health == 0 then
    mod_print("I am dead: %s", self.display_name)
    self:SetCommand("Dead")
  end
  if not IsValid(self.last_attacker) then
    self.last_attacker = nil
  elseif IsKindOf(self.last_attacker, "Building") and self.last_attacker.destroyed then
    self.last_attacker = nil
  elseif IsKindOf(self.last_attacker, "AttackRover") and self.last_attacker:IsDead() then
    self.last_attacker = nil
  end
  self.last_attacker = self.last_attacker or shooter
  ObjModified(self)
  if self.command == "Roam" then
    self:SetCommand("Idle")
  end
end

function DefenceTower:DefenceTick()
  local hostile = self.city.labels.HostileAttackers or empty_table
  if IsValidThread(self.track_thread) then
    return
  end
  -- mod_print(#hostile)
  for i = 1, #hostile do
    local obj = hostile[i]
    if self:GetVisualDist2D(obj) <= self.shoot_range then
      self:OrientPlatform(obj:GetVisualPos(), 7200)
      if not IsValid(self) or not self.working or self.destroyed then
        return
      end
      self:FireRocket(nil, obj)
      Sleep(self.reload_time)
      self.is_firing = false
      return true
    end
  end
end

function RocketProjectile:HitTarget(target)
  if IsKindOf(target, "Dome") then
  elseif IsKindOf(target, "Building") then
    PlayFX("MeteorDestruction", "start", target)
    -- DestroyBuildingImmediate(target)
    -- table.insert_unique(g_AIDestroyedBuildings, target)
	target:AccumulateMaintenancePoints(2000)
	if target:GetMaintenanceProgress() >= 100 or not target.accumulate_dust or not target:CanDemolish() or target.indestructible then
		DestroyBuildingImmediate(target)
		table.insert_unique(g_AIDestroyedBuildings, target)
	end
  elseif IsKindOf(target, "AttackRover") then
    PlayFX("MeteorDestruction", "start", target)
    target:RocketDamage(self.shooter)
  elseif IsKindOf(target, "DroneBase") then
    PlayFX("MeteorDestruction", "start", target)
    if not target:IsDead() then
      target:SetCommand("Dead")
    end
  elseif IsKindOf(target, "Colonist") then
    PlayFX("MeteorDestruction", "start", target)
    if not target:IsDead() then
      target:SetCommand("Dead")
    end
  end
end

function BaseAttacker:HitTarget(target)
  if IsKindOf(target, "Dome") then
  elseif IsKindOf(target, "Building") then
    PlayFX("MeteorDestruction", "start", target)
    -- DestroyBuildingImmediate(target)
    -- table.insert_unique(g_AIDestroyedBuildings, target)
	-- 根据攻击力计算
	target:AccumulateMaintenancePoints(self.rocket_damage * 300)
	if target:GetMaintenanceProgress() >= 100 or not target.accumulate_dust or not target:CanDemolish() or target.indestructible then
		DestroyBuildingImmediate(target)
		table.insert_unique(g_AIDestroyedBuildings, target)
	end
  elseif IsKindOf(target, "AttackRover") then
    PlayFX("MeteorDestruction", "start", target)
    target:RocketDamage(self)
  elseif IsKindOf(target, "DroneBase") then
    PlayFX("MeteorDestruction", "start", target)
    if not target:IsDead() then
      target:SetCommand("Dead")
    end
  elseif IsKindOf(target, "Colonist") then
    PlayFX("MeteorDestruction", "start", target)
    if not target:IsDead() then
      target:SetCommand("Dead")
    end
  end
end

function Drone:Done()
  self:Gossip("done")
  self.city:RemoveFromLabel("Unit", self)
  self.city:RemoveFromLabel("AttackDrone", self)
  self.city:RemoveFromLabel("RogueDrones", self)
  self.city:RemoveFromLabel("HostileAttackers", self)
  UpdateRogueNotification()
  if self.is_orphan then
    table.remove_entry(g_OrphanedDrones, self)
  elseif self.command_center and IsValid(self.command_center) then
    table.remove_entry(self.command_center.drones, self)
  end
end

function Drone:RogueRevertToNormal()
  if not self.rogue or self:IsDead() then
    return
  end
  self.city:RemoveFromLabel("RogueDrones", self)
  self.city:RemoveFromLabel("HostileAttackers", self)
  UpdateRogueNotification()
  self.rogue = nil
  self:SetMoveAnim("moveWalk")
  self:SetWaitAnim("idle")
  local signs = self:GetAttaches("UnitSignRogue")
  for i = 1, #signs do
    DoneObject(signs[i])
  end
  self:SetCommandCenter(false)
  RebuildInfopanel(self)
end

function Drone:Dead(already_dead)
  self:PushDestructor(function(self)
  end)
  self.city:RemoveFromLabel("RogueDrones", self)
  self.city:RemoveFromLabel("HostileAttackers", self)
  UpdateRogueNotification()
  if not already_dead then
    RebuildInfopanel(self)
    self:StartFX("Dead")
    self:ClearPath()
    self:SetPos(self:GetVisualPos())
    self:PlayState("death")
    self:SetIsNightLightPossible(false)
    self:SetCommandCenter(false)
  end
  Halt()
end

function Drone:GoRogue()
	if self.rogue or self:GetParent() then
		return
	end
	self.city:AddToLabel("RogueDrones", self)
	self.city:AddToLabel("HostileAttackers", self)
	UpdateRogueNotification()
	self.rogue = true
	self:SetCarriedResource(false)
	if IsValid(self.command_center) then
		table.remove_entry(self.command_center.drones, self)
		self.command_center = nil
	end
	self:SetMoveAnim("rogueWalk")
	self:SetWaitAnim("rogueIdle")
	local sign = AttachToObject(self, "UnitSignRogue", "Origin")
	sign:SetVisible(true)
	sign:SetScale(const.SignsOverviewCameraScaleDown)
	self.MoveSleep = self.RogueMoveSleep
	RebuildInfopanel(self)
end

function Drone:RogueAttack(target)
  if not IsValid(target) then
    local target_label = self:Random(100) < self.rogue_attack_colonist_chance and "Colonist" or "Life-Support"
    local list = self.city.labels[target_label] or empty_table
    if #list == 0 then
      return self:RogueWander()
    end
    target = self.city:TableRand(list)
  end
  if not IsValid(target) then
    return
  end
  local is_unit = IsKindOf(target, "Colonist")
  local is_building = IsKindOf(target, "Building")
  self.rogue_intercepted = self.rogue_intercepted or {}
  self:StartMoving()
  self:PushDestructor(function(self)
    self.rogue_target = nil
  end)
  self.rogue_target = target
  self:GotoSameDomeAsObj(target)
  self:Goto(target, 2 * guim, 0)
  self:PopAndCallDestructor()
  if not IsValid(target) or is_unit and target:IsDying() or is_building and target.destroyed then
    return
  end
  if is_unit then
    if HexAxialDistance(self, target) > 1 then
      return
    end
    self:Face(target)
    self:SetAnim(1, "rogueAttack")
    target:SetCommand("AttackedByDrone", self)
    Sleep(self:TimeToAnimEnd())
    if IsValid(target.current_dome) then
      target.current_dome:AddPanicMarker(target:GetPos(), 15 * guim, 3 * const.HourDuration)
    end
  else
    local shape = GetEntityOutlineShape(target:GetEntity())
    local in_range = false
    local angle = target:GetAngle()
    local origin = target:GetPos()
    for i = 1, #shape do
      local q, r = shape[i]:xy()
      local x, y = HexToWorld(q, r)
      local dir = Rotate(point(x, y), angle)
      local pt = origin + dir
      if 2 > HexAxialDistance(self, pt) then
        in_range = true
        break
      end
    end
    if not in_range then
      return
    end
    PlayFX("DroneExplode", "start", self)
    DestroyBuildingImmediate(target)
    table.insert(g_AIDestroyedBuildings, target)
    self:SetCommand("Dead")
  end
end

function DroneBase:ToggleAttackMode()
  if self.interaction_mode == "attack" then
    SetUnitControlInteractionMode(self, false)
  else
    SetUnitControlInteractionMode(self, "attack")
    SetUnitControlFocus(true, self)
  end
end

function DroneBase:ToggleRoamMode()
  self.allow_roam =  not self.allow_roam
  if self.allow_roam then 
	 self:SetCommand("Attack", -1) 
  else
     self:SetCommand("Idle")
  end
  if self.interaction_mode == "roam" then
    SetUnitControlInteractionMode(self, false)
  else
    SetUnitControlInteractionMode(self, "roam")
    SetUnitControlFocus(true, self)
  end
end

function DroneBase:ToggleAttackMode_Update(button)
  local to_mode = self.interaction_mode ~= "attack"
  button:SetIcon(to_mode and "UI/Icons/IPButtons/shuttle.tga" or "UI/Icons/IPButtons/cancel.tga")
  button:SetRolloverTitle(T{"攻击"})
  button:SetRolloverText(T{"选择一个敌人并攻击他"})
  if not to_mode or not T({
    7509,
    "<left_click> Select target mode"
  }) then
  end
  button:SetRolloverHint((T({
    7510,
    "<left_click> on target to select it  <right_click> Cancel"
  })))
  if not to_mode or not T({
    7511,
    "<ButtonA> Select target mode"
  }) then
  end
  button:SetRolloverHintGamepad((T({
    7512,
    "<ButtonA> Cancel"
  })))
  button:SetEnabled(self:CanBeControlled())
end

function DroneBase:ToggleRoamMode_Update(button)
  
  local to_mode = self.allow_roam
  button:SetIcon(not to_mode and "UI/Icons/IPButtons/shuttle.tga" or "UI/Icons/IPButtons/cancel.tga")
  button:SetRolloverTitle(T{"自动漫游"})
  button:SetRolloverText(T{"自动漫游并且寻找敌人"})
  button:SetEnabled(self:CanBeControlled())
end


function OnMsg.ClassesBuilt()
	Drone.rogue_shoot_range = 40 * guim
	Drone.rogue_shoot_chance = 60
	
	table.insert(XTemplates.ipAttackRover[1], 
        PlaceObj("XTemplateTemplate", {
			"comment",
			"move",
			"__condition",
			function(parent, context)
				return context.reclaimed and not context:CanBeRepaired()
			end,
			"__template",
			"InfopanelButton",
            "Icon", "UI/Icons/Upgrades/factory_ai_02.tga",
            "RolloverTitle", T{"攻击敌人"},
            "OnPressParam",
            "ToggleAttackMode"
        })
    )
	table.insert(XTemplates.ipAttackRover[1], 
        PlaceObj("XTemplateTemplate", {
			"comment",
			"move",
			"__condition",
			function(parent, context)
				return context.reclaimed and not context:CanBeRepaired()
			end,
			"__template",
			"InfopanelButton",
            "Icon", "UI/Icons/Upgrades/factory_ai_02.tga",
            "RolloverTitle", T{"自动漫游"},
            "OnPressParam",
            "ToggleRoamMode"
        })
    )
end

-- function OnMsg.ClassesBuilt()
	-- AttackAddInfoSection()
    -- PlaceObj("Cargo", {
		-- description = T({
		-- 5309,
		-- "Recycles up to 70% of the Water used in the Dome."
		-- }),
		-- group = "Rovers",
		-- id = "AttackRover",
		-- name = T{"超级战车"},
		-- price = 500000000,
		-- kg = 20000,
	-- })
-- end

-- DefineClass.AttackBase = {
	-- reload_time = const.HourDuration / 10,
	-- reload_expiration_time = false,
	-- reclaimed = false,
	-- max_health = 100,
	-- attack_damage = 5,
	-- current_health = false,
	-- attacks_remaining = false,
	-- last_attacker = false,
	-- use_random_pos = true,
	-- spawn_min_angle = 1800,
	-- spawn_max_angle = 4200,
	-- spawn_flight_dist = 1500 * guim,
	-- spawn_land_dist = 300 * guim,
	-- spawn_flight_speed = 150 * guim,
	-- spawn_pos = false,
	-- track_thread = false,
	-- intercept_distance_percent = 80,
	-- intercept_reload_percent = 10,
	-- is_repair_request_initialized = false,
	-- attack_look_for_target = false,
	-- land_decal_name = "DecRocketSplatter"
-- }



function AttackRover:Spawn()
  local city = self.city or UICity
  local spawn_pos = self.spawn_pos
  self:PushDestructor(function()
    local dir = point(city:Random(-4096, 4096), city:Random(-4096, 4096))
    local angle = city:Random(self.spawn_min_angle, self.spawn_max_angle)
    local s, c = sin(angle), cos(angle)
    local flight_dist = self.spawn_flight_dist
    if c == 0 then
      dir = point(0, 0, flight_dist)
    else
      dir = SetLen(dir:SetZ(MulDivRound(dir:Len2D(), s, c)), flight_dist)
    end
    while not spawn_pos do
      local sector_x = city:Random(1, 10)
      local sector_y = city:Random(1, 10)
      local sector = g_MapSectors[sector_x][sector_y]
      local minx, miny = sector.area:minxyz()
      local maxx, maxy = sector.area:maxxyz()
      local x = city:Random(minx, maxx)
      local y = city:Random(miny, maxy)
      local pt = point(x, y)
      if IsPlayablePoint(pt) and not GetDomeAtPoint(pt) then
        pt = pt:SetStepZ()
        if not DomeCollisionCheck(pt, pt + dir) then
          spawn_pos = pt
          break
        end
      end
    end
    local pos = spawn_pos + dir
    local rover = PlaceObject("FakeAttackRover")
    local palette = self.palettes and EntityPalettes[self.palettes[1]]
    if palette then
      SetObjectPalette(rover, palette)
    end
    rover:SetPos(pos)
    PlayFX("Meteor", "start", rover, nil, nil, dir)
    spawn_pos = terrain.GetIntersection(pos, spawn_pos)
    flight_dist = spawn_pos:Dist(pos)
    local flight_speed = self.spawn_flight_speed
    local total_time = MulDivRound(1000, flight_dist, flight_speed)
    local land_time = MulDivRound(1000, self.spawn_land_dist, flight_speed)
    local pitch = -atan(dir:Len2D(), dir:z())
    local yaw = 10800 + atan(dir:y(), dir:x())
    SetRollPitchYaw(rover, 0, pitch, yaw, 0)
    rover:SetPos(spawn_pos, total_time)
    Sleep(total_time - land_time)
    PlayFX("Land", "start", rover)
    local final_speed = 10 * guim
    local accel, land_time = rover:GetAccelerationAndTime(spawn_pos, final_speed, flight_speed)
    rover:SetAcceleration(accel)
    rover:SetPos(spawn_pos, land_time)
    SetRollPitchYaw(rover, 0, 0, yaw, land_time)
    Sleep(land_time)
    PlayFX("Land", "end", rover)
    PlayFX("Meteor", "end", rover, nil, nil, dir)
    local land_decal = PlaceObject(self.land_decal_name)
    land_decal:SetPos(spawn_pos)
    land_decal:SetAngle(AsyncRand(21600))
    land_decal:SetScale(15)
    CreateGameTimeThread(function()
      local delta = const.DayDuration / 20
      for opacity = 100, 0, -5 do
        Sleep(delta)
        if not IsValid(land_decal) then
          return
        end
        land_decal:SetOpacity(opacity, delta)
      end
      DoneObject(land_decal)
    end)
    rover:SetAnimSpeedModifier(1000 + MulDivRound(500, final_speed, 5 * guim))
    rover:PlayState("attackStart")
    rover:SetAnimSpeedModifier(1000)
    rover:PlayState("attackEnd")
    if IsValid(self) then
      self:SetPos(spawn_pos)
      self:SetAngle(rover:GetAngle())
      self:SetAxis(rover:GetAxis())
    end
    DoneObject(rover)
    Sleep(1000)
  end)
  self:PopAndCallDestructor()
  if self.reclaimed then
	self:SetCommand("Attack", -1)
  else
	self:SetCommand("Roam")
  end
end


