-- 本代码作者是Alexliyu, 依据游戏源代码以及开发文档进行编写, 并遵守与继承游戏本身的协议.如果需要二次修改,提取或者分发请注明出处
-- GlobalVar("g_DangerPeriodicSaveThread", false)
DangerPeriodicSaveThread = false
DefineClass.DangerBase = {
	attackRover_attack_range = 750 * guim,
	attackDrone_per_cost = 1000000000,
	attackRover_per_cost = 3000000000,
	attackFemal_per_cost = 5000000000,
	difficulty_level = 3,
	-- probabilities
	-- 每日每个穹顶得病几率
	sickness_chance_infirmary = 10,
	sickness_chance_no_infirmary = 30,

	-- independent variables (medial services remove sickness)
	-- daily
	sickness_spread_chance = 66,
	sickness_self_cure_chance = 33,

	-- only spread : spread_chance% * (1 - self_cure_chance%)
	-- only cure : self_cure_chance% * (1 - spread_chance%)
	-- neither spread nor self cure (1 - spread_chance%)*(1 - self_cure_chance%)
	-- both : spread_chance% * self_cure_chance%

	sickness_daily_health_loss = 10,		-- effect every day
	sickness_daily_performance_loss = 10, -- effect every hour

	sickness_loss = 10, -- both performance and health loss by default
	detrimental_status_effects = {
	"StatusEffect_Irradiated",
	"StatusEffect_Suffocating",
	"StatusEffect_Dehydrated",
	"StatusEffect_Freezing" ,
	"StatusEffect_Starving",
	"StatusEffect_StressedOut",
	"StatusEffect_Earthsick",
	"StatusEffect_Sick",
	},

	incident_data = {
	  devil = 0,
	  bombard = 0,
	  renegade = 0,
	  attack = 0,
	  hostiledead = 0,
	}
}
DangerBase.StringIdBase = 82827346

local function GetDangerSaveFolder()
	return "AppData/DangerSave"
end

-- 生成保存数据的文件名称
local function GenerateFilename(latitude, longitude)
	-- 文件名取决于火星坐标，如果没有提供，我们将提取它们。
	if not (latitude and longitude) then
		-- 我们不能从地图中提取坐标，如果它不是一个游戏地图。
		-- (还有其他类型的地图，由于技术原因不运行游戏逻辑。)
		if mapdata and mapdata.GameLogic then
			-- 简单地从当前映射参数提取坐标。
			latitude = g_CurrentMapParams.latitude
			longitude = g_CurrentMapParams.longitude
		end
	end
	if latitude and longitude then
		return string.format("%s/danger_%d_%d.lua", GetDangerSaveFolder(), latitude, longitude)
	end
end

-- 抛出一个线程， 每五分钟保存一次数据
local function PeriodicSave()
	if not (mapdata and mapdata.GameLogic) then
		return
	end
	Sleep(5*60*1000)
	
	while mapdata.GameLogic do
		DangerBase:Save()
		Sleep(5*60*1000)
	end
end

-- 获取对象列表
function DangerBase:GetObjectsByLabel(label)
  if label == "Working-age" then
    local ret = {}
    for _, dome in ipairs(UICity.labels.Dome or empty_table) do
      table.append(ret, dome.labels.Youth)
      table.append(ret, dome.labels.Adult)
      table.append(ret, dome.labels["Middle Aged"])
    end
    return ret
  else
    return UICity.labels[label]
  end
end
	
-- 初始化
function DangerBase:Init()
	-- 避免线程运行两次
	if not IsValidThread(DangerPeriodicSaveThread) then
		DangerPeriodicSaveThread = CreateRealTimeThread(PeriodicSave)
	end
	self:ShowStartNotification()
    self:AddShopScreen()
	self:AddIncidentScreen()
	-- 提供初始资源
    if not UICity.labels.HostileAttackers or #UICity.labels.HostileAttackers < 1 then
		CreateGameTimeThread(function()
			self:SpawnAttackDrone(3)
			self:SpawnAttackRover(2)
			self:SpawnAttackFemale(1)
			self:SpawnHostileAttackFemale(2)
			self:SpawnHostileAttackDrone(3)
		end)
    end
   -- PlaceObj("TechEffect_UnlockResupplyItem", {"Item", "AttackRover"})
end

function DangerBase:InitMystery()
	GrantTech("DefenseTower")
    if not UICity.mystery then
       UICity.mystery = MysteryBase
    end
	UICity.mystery.can_shoot_rovers = true
    UICity.mystery.enable_rover_repair = true
    UICity.mystery.reclaim_repaired_rovers = true
end

-- 加载之前的数据.
function DangerBase:Load(filename)
    local load_data
	local file_error
	local code
	local code_error
	filename = filename or GenerateFilename()
	if not filename then return end
	file_error, code = AsyncFileToString(filename)
	if file_error then return end
	mod_print("读取数据:%s", code)
	code_error, load_data = LuaCodeToTuple(code)
	if code_error then return end
	-- 填充数据
	mod_print("尝试填充数据%s", code_error)
	for k, v in pairs(load_data) do
	    mod_print("key is %s,%s", k, v)
		self.incident_data[k] = v
	end
end

-- 保存数据到磁盘.
function DangerBase:Save(filename)
    if not self.incident_data then return end
    local mod_data = self.incident_data or {}
	filename = filename or GenerateFilename()
	if not filename then
		return
	end
	
	local folder_error = AsyncCreatePath(GetDangerSaveFolder())
	if folder_error then
		return folder_error
	end
	-- local code = ValueToLuaCode(mod_data)
	local code = ValueToLuaCode(mod_data)
	
	local file_error = AsyncStringToFile(filename, code)
	return file_error
end

-- 无人机开始随机发疯
function DangerBase:DronesRogue(number)
  self.incident_data.attack = self.incident_data.attack + 1
  local drones = self:GetObjectsByLabel("Drone") or {}
  if number == -1 or number >= #drones then
    for _, drone in ipairs(drones) do
      drone:SetCommand("GoRogue")
    end
  else
    local affected = {}
    for i = 1, number do
      local r
      repeat
        r = 1 + UICity:Random(#drones)
      until not affected[r]
      affected[r] = true
    end
    for i, _ in pairs(affected) do
      drones[i]:SetCommand("GoRogue")
    end
  end
end

-- 无人机开始恢复正常状态
function DangerBase:DronesNoraml()
 local drones = self:GetObjectsByLabel("Drone")
	for _, drone in ipairs(drones) do
		if IsValid(drone) then
			drone:SetCommand("RogueRevertToNormal")
		end
	end
end

-- 战车开始进行攻击
function DangerBase:StartHostileAttackRoverAttack()
	local drones = self:GetObjectsByLabel("HostileAttackers")
	for _, drone in ipairs(drones) do
		if not drone.reclaimed then
		    drone:SetCommand("Reload")
			drone:SetCommand("Attack", -1)
		end
	end
end

-- 添加敌对战车
function DangerBase:SpawnHostileAttackRover(number)
    self.incident_data["attack"] = self.incident_data["attack"] + 1
    AddCustomOnScreenNotification("dangerAttackRover", 
		T{"警报"}, T{"<number> 个战车即将进入大气层",number=number}, "UI/Icons/Notifications/rocket.tga", 
		false,
		{
			expiration = 15000,
			priority = "Important",
		})	
    for i = 1, number do
		PlaceObject("MyAttackRover", {
			use_random_pos = true,
			can_repair = true,
			reclaimed = false,
			need_spawn = true,
		})
		
    end
    -- rover2:SetCommand("Attack", 3)
end

-- 添加玩家战车
function DangerBase:SpawnAttackRover(number)
	for i = 1, number do
		local rover = PlaceObject("MyAttackRover", {
		  use_random_pos = true,
		  can_repair = true,
		  reclaimed = true,
		  need_spawn = true,
		  palettes = {"AttackRoverRed"}
		})
		UICity:RemoveFromLabel("HostileAttackers", rover)
		-- UICity:AddToLabel("Rover", rover)
		CommandObject.SetCommand(rover, "Idle")
	end
end

-- 添加玩家战斗机器人
function DangerBase:SpawnAttackDrone(number)

	for i = 1, number do
		local rover = PlaceObject("MyAttackDrone", {
		  use_random_pos = true,
		  can_repair = true,
		  reclaimed = true,
		  need_spawn = true,
		  palettes = {"AttackRoverRed"}
		})
		UICity:RemoveFromLabel("HostileAttackers", rover)
		-- UICity:AddToLabel("Rover", rover)
		CommandObject.SetCommand(rover, "Idle")
	end
end

-- 添加敌对战斗机器人
function DangerBase:SpawnHostileAttackDrone(number)
    self.incident_data["attack"] = self.incident_data["attack"] + 1
	AddCustomOnScreenNotification("dangerAttackRogueDrone", 
		T{"警报"}, T{"<number> 个战斗无人机即将进入大气层",number=number}, "UI/Icons/Notifications/rocket.tga", 
		false,
		{
			expiration = 15000,
			priority = "Important",
		})
		for i = 1, number do
	      local rover = PlaceObject("MyAttackDrone", {
		      use_random_pos = true,
			  can_repair = true,
			  reclaimed = false,
			  need_spawn = true,
			  pin_on_start = false,
		    })
		end
    -- local w, h = terrain.GetMapSize()
    -- local center = GetPassablePointNearby(point(w / 2, h / 2))		
	-- for i = 1, number do
	    -- local drone = PlaceObject("Drone", {
		      -- notReclaimed = true,
			  -- pin_on_start = true,
			  -- rogue_shoot_range = 60 * guim,
			  -- rogue_shoot_chance = 40,
			  -- land_decal_name = "DecRocketSplatter",
			  -- entity = "Drone_Trailblazer",
			  -- default_skins = {
				-- "Drone_Trailblazer"
			  -- }
		    -- })
		-- local pos = drone:GetRandomPos(30 * guim, 2 * guim, center)
        -- drone:SetPos(pos or center)
		-- drone:SetCommand("GoRogue")
	-- end
end

-- 添加玩家魔法师
function DangerBase:SpawnAttackFemale(number)
	for i = 1, number do
		local rover = PlaceObject("MyAttackFemale", {
		  use_random_pos = true,
		  can_repair = true,
		  reclaimed = true,
		  need_spawn = true,
		  pin_on_start = true,
		  palettes = {"AttackRoverRed"}
		})
		UICity:RemoveFromLabel("HostileAttackers", rover)
		-- UICity:AddToLabel("Rover", rover)
		CommandObject.SetCommand(rover, "Idle")
	end
end

-- 添加敌对魔法师
function DangerBase:SpawnHostileAttackFemale(number)
    self.incident_data.attack = self.incident_data.attack + 1
	AddCustomOnScreenNotification("dangerAttackFemale", 
		T{"警报"}, T{"<number> 个阿修罗即将进入大气层",number=number}, "UI/Icons/Notifications/rocket.tga", 
		false,
		{
			expiration = 15000,
			priority = "Important",
		})
		for i = 1, number do
	      local rover = PlaceObject("MyAttackFemale", {
		      use_random_pos = true,
			  can_repair = true,
			  reclaimed = false,
			  need_spawn = true,
			  pin_on_start = false,
		    })
		end
end


-- 购买战斗兵器
function DangerBase:BuyAttackDrones(number)
	if number == 1 then
		UICity:ChangeFunding(-self.attackRover_per_cost)
		self:SpawnAttackRover(1)
	elseif number == 2 then
		UICity:ChangeFunding(-self.attackDrone_per_cost)
		self:SpawnAttackDrone(1)
	elseif number == 3 then
		UICity:ChangeFunding(-self.attackFemal_per_cost)
		self:SpawnAttackFemale(1)
	end
end

function DangerBase:ShowStartNotification()
	CreateRealTimeThread(WaitCustomPopupNotification,
		T{"开启危险模式"},
		T{"尊敬的指挥官，您已经开启了危险模式，因此10天后将会有源源不断的攻击到来. <newline>我赠送了您3辆火箭战车并且帮您开启了防御炮塔，请努力活过120天吧.<newline>祝您好运，哈哈",},
		{ T{"如果您120天后仍旧幸存，请告知我. 我是Alexliyu, 这个世界的魔王"} })
end

function DangerBase:ShowOverNotification()
	CreateRealTimeThread(function()
	    local title = T{"恭喜"}
		local text = T{"您是一个真正的英雄, 这里已经没有任何事情能阻挡您的脚步，我将会离开，祝好运.",}
		local choice = {
			T{"继续"},
			T{"再见"},
		}
		local res = WaitCustomPopupNotification(title, text , choice)
		    if res == 2 then
			CreateRealTimeThread(function()
			  OpenPreGameMainMenu()
			end)
		    end
	end)
end

-- 显示战斗机器人商店
function DangerBase:ShowAttackDroneShop()
	CreateRealTimeThread(function()
	    local title = T{"欢迎光临火星大魔王军火市场"}
		local text = T{"尊敬的指挥官, 请问您想要购买什么？",}
		local choice = {
			T{"我需要战车--攻击力5,生命300,移动速度慢的远程攻击武器 <right>价格<colonist(number)>",number=3000},
			T{"我需要无人机--攻击力25,生命100,强悍快速的近战武器 <right>价格<colonist(number)>",number=1000},
			T{"我需要阿修罗--攻击力15,生命100,可操纵雷电进行群体攻击的法师 <right>价格<colonist(number)>",number=5000},
			T{"不买啦奸商"},
		}
		local res = WaitCustomPopupNotification(title, text , choice)
		    if res < 4 then
				self:BuyAttackDrones(res)
		    end
	end)
end

-- 显示作战统计
function DangerBase:ShowIncidentScreen()
    
	CreateRealTimeThread(function()
		 -- devil = 0,
	  -- bombard = 0,
	  -- renegade = 0,
	  -- attack = 0,
	  -- hostiledead = 0,
	    local title = T{"您当前的作战情况如下"}
		local texts = {
			T{"<center><em>遭遇</em>"},
			T{"遭遇自然灾害次数: <right><colonist(number)>",number=self.incident_data.devil or 0},
			T{"遭遇流星打击次数: <right><colonist(number)>",number=self.incident_data.bombard or 0},
			T{"遭遇小人叛变次数: <right><colonist(number)>",number=self.incident_data.renegade or 0},
			T{"遭遇魔王攻击次数: <right><colonist(number)>",number=self.incident_data.attack or 0},
			T{"<center><em>战果</em>"},
			T{"总共杀死敌人数量: <right><colonist(number)>",number=self.incident_data.hostiledead or 0},
			
		}
		local text = table.concat(texts, "<newline><left>")
		local choice = {
			T{"关闭"},
		}
		local res = WaitCustomPopupNotification(title, text , choice)
	end)
end

-- 添加购买商店到屏幕
function DangerBase:AddShopScreen()
    RemoveOnScreenNotification("OrderAttackDron")
	AddCustomOnScreenNotification("OrderAttackDron", 
			T{"购买战斗无人机"}, T{"点击本图标购买战斗无人机"}, "UI/Icons/Notifications/funding.tga", 
			function()
				self:ShowAttackDroneShop()
			end,
			{
				priority = "Critical",
				dismissable = false,
			}
	)
end
-- 添加各种数据到屏幕
function DangerBase:AddIncidentScreen()
    RemoveOnScreenNotification("ShowIncidentScreen")
	AddCustomOnScreenNotification("ShowIncidentScreen", 
			T{"作战统计"}, T{"点击本图标显示作战统计"}, "UI/Icons/Notifications/meteor_storm_2.tga", 
			function()
				self:ShowIncidentScreen()
			end,
			{
				priority = "Critical",
				dismissable = false,
			}
	)
end

-- 开始随机的自然灾害
function DangerBase:StartRandDevil()
    self.incident_data.devil = self.incident_data.devil + 1
	local number = UICity:Random(4)
	if number == 0 then
		CheatDustDevil("great")
	elseif number == 1 then
		CheatDustDevil("electrostatic")
	elseif number == 2 then
		CheatColdWave()
	elseif number == 3 then
		CheatColdWave()
		CheatDustDevil("electrostatic")
	else
		CheatDustDevil("great")
		CheatColdWave()
		CheatDustDevil("electrostatic")
	end
end

-- 产生随机的叛徒
function DangerBase:SpawnRenegadeNColonists(number)
  self.incident_data.renegade = self.incident_data.renegade + 1
  if number == -1 or number >= #UICity.labels.Colonist then
    for _, colonist in ipairs(UICity.labels.Colonist) do
      colonist:AddTrait("Renegade")
    end
  else
    local affected = {}
    for i = 1, number do
      local r
      repeat
        r = 1 + UICity:Random(#UICity.labels.Colonist)
      until not affected[r]
      affected[r] = true
    end
    for i, _ in pairs(affected) do
      UICity.labels.Colonist[i]:AddTrait("Renegade")
    end
  end
end

function DangerBase:StartHostileAttackRoverAttackDelay(delay_time)
	DelayedCall(delay_time, function()
		DangerBase:StartHostileAttackRoverAttack()
	end)
end

function DangerBase:DronesNoramlDelay(delay_time)
	DelayedCall(delay_time, function()
		DangerBase:DronesNoraml()
	end)
end

-- self cure, spread
function DangerBase:SpreadOrCure(colonist)
	-- spread
	if colonist.dome then
		if colonist:Random(100) <= DangerBase.sickness_spread_chance then
			local colonists = colonist.dome.labels.Colonist
			local n = #colonists
			local start_idx = colonist:Random(n) + 1
			for i = 1, n do
				local unit = colonists[ (start_idx + i) % n + 1]
				if not unit.status_effects.StatusEffect_Sick then
					unit:Affect("StatusEffect_Sick", "start")
					break
				end
			end
			
			-- passages
			
		end
	end
	
	-- cure
	if colonist:Random(100) <= DangerBase.sickness_self_cure_chance then
		colonist:Affect("StatusEffect_Sick", false)
	end
end

DefineClass.StatusEffect_Sick =  {
	__parents = { "StatusEffect"},
	display_name = T{"魔王诅咒"},
	description = T{"每天失去生命,并且可以传染给其他殖民者.生病的殖民者随着时间的推移会降低效率."},
	sign = "UnitSignIrradiated",
	selection_arrow = "UnitArrowIrradiated",
	priority = 5,
	reason = T{582972083982, "<red>魔王诅咒 <amount></red>"},
}

function StatusEffect_Sick:Start(unit, start)
	if DangerBase.sickness_loss ~= 1 then
		unit:SetModifier("performance", "StatusEffect_Sick", -DangerBase.sickness_daily_performance_loss, 0, self.reason)
	end
	unit:AddTrait("Sick")
end

function StatusEffect_Sick:Stop(unit)
	unit:SetModifier("performance", "StatusEffect_Sick", 0, 0)
	unit:RemoveTrait("Sick")
end

function StatusEffect_Sick:HourlyUpdate(unit, start, hours)
	if DangerBase.sickness_loss ~= 1 then
		unit:SetModifier("performance", "StatusEffect_Sick", -tonumber(string.format("%.1f",DangerBase.sickness_daily_performance_loss * (1 + (hours * 1.0) / const.HoursPerDay))), 0, self.reason)
	end
end

function StatusEffect_Sick:DailyUpdate(unit, start, hours)
	if DangerBase.sickness_loss ~= 3 then
		unit:ChangeHealth(-DangerBase.sickness_daily_health_loss * const.Scale.Stat, self.class)
	end
	DangerBase:SpreadOrCure(unit)
end

function GetDetrimentalStatusEffects()
	return DangerBase.detrimental_status_effects
end

function OnMsg.LoadGame()
	ColonistStatReasons["StatusEffect_Sick"] = StatusEffect_Sick.reason
	CreateGameTimeThread(function()
		-- 再次检查是否获取了地图数据
		if mapdata and mapdata.GameLogic then
			-- 尝试从以前数据中获取历史数据
			mod_print("尝试从以前数据中获取历史数据")
			DangerBase:Load()
		end
	end)
	DelayedCall(10000, function()
		DangerBase:InitMystery()
		DangerBase:Init()
	end)
end

function OnMsg.GameTimeStart()
	ColonistStatReasons["StatusEffect_Sick"] = StatusEffect_Sick.reason
end

-- medical services remove sickness
VanillaMedicalBuildingService = MedicalBuilding.Service
-- overwrite vanilla medical services service
function MedicalBuilding:Service(unit, duration, daily_interest)
	ServiceWorkplace.Service(self, unit, duration)
	if daily_interest == "interestRelaxation" and self:IsRejuvenationTreatment() then
		unit:ChangeComfort( TechDef.RejuvenationTreatment.param1 * const.Scale.Stat, self.template_name )
	end
	if unit.traits.Infected and g_StartVaccinating then
		unit:RemoveTrait("Infected")
	end
	
	-- will also remove sick trait from unit
	unit:Affect("StatusEffect_Sick", false)
end

function SickOnDay(day)
for _, dome in ipairs(UICity.labels.Dome or empty_table) do		
		for _, workplace in ipairs(dome.labels.Workplaces or empty_table) do			
			local workers_outside_at_night = {}
			local workers_outside = {}
			
			local is_outside_building = not workplace.parent_dome
			if is_outside_building then
				local from, to = workplace.active_shift > 0 and workplace.active_shift or 1, workplace.active_shift > 0 and workplace.active_shift or workplace.max_shifts
				
				local occupied = workplace.workers or empty_table
				
				for i = from, Min(to, 2) do
					for _, worker in ipairs(occupied[i] or empty_table) do
						table.insert(workers_outside, worker)
					end
				end
				
				-- night shift
				if to == 3 then
					for _, worker in ipairs(occupied[3] or empty_table) do
						table.insert(workers_outside_at_night, worker)
					end
				end
							
				local chance = DangerBase.sickness_chance_no_infirmary
				-- has medical building ?
				if dome.labels.needMedical and #dome.labels.needMedical > 0 then
					chance = DangerBase.sickness_chance_infirmary
				end
				
				local proba = dome:Random(100)
				local found = false
				
				-- x2 if cold wave
				if g_ColdWave then
					chance = 2 * chance
				end
				
				-- x1.5 if outside during night hours (becomes x3 base chance with cold wave)
				if next(workers_outside_at_night) and proba <= 1.5 * chance then
					for _, worker in ipairs(workers_outside_at_night) do
						if not worker.status_effects.StatusEffect_Sick then
							worker:Affect("StatusEffect_Sick", "start")
							found = true
							break
						end
					end			
				end
				
				if next(workers_outside) and proba <= chance and not found then
					for _, worker in ipairs(workers_outside) do
						if not worker.status_effects.StatusEffect_Sick then
							worker:Affect("StatusEffect_Sick", "start")
							found = true
							break
						end
					end
				end
			end -- end if outside workplace
		end -- end for all workplaces
	end -- end for all domes
end

function OnMsg.PostNewMapLoaded()
	DelayedCall(10000, function()
		DangerBase:InitMystery()
		DangerBase:Init()
	end)
    
end

function OnMsg.MysteryChosen()
    DangerBase:InitMystery()
end

function OnMsg.SaveGame()
	if mapdata and mapdata.GameLogic then
		DangerBase:Save()
	end
end

-- 当有小孩出生
function OnMsg.ColonistBorn(colonist, event)
   local rand = colonist:Random(100)
   if event == "born" and rand < 30 and  UICity.day > 20 and UICity.labels.Dome then
      local dome_id = UICity:Random(#UICity.labels.Dome)
      StartBombard(UICity.labels.Dome[dome_id], 500*guim, 20, 2000, 6000)
	  DangerBase.incident_data.bombard = DangerBase.incident_data.bombard + 1
   end
   if event == "born" and rand < 10 and  UICity.day > 20 then
      DangerBase:SpawnRenegadeNColonists(5)
   end
   if event == "born" and rand < 5 and  UICity.day > 20 then
      DangerBase:StartRandDevil()
   end
end

-- 当有乘客到达
function OnMsg.ColonistArrived(colonist)
  if colonist:Random(100) < 10 and  UICity.day > 30 then
      DangerBase:SpawnHostileAttackRover(2)
      DangerBase:DronesRogue(2)
	  DangerBase:SpawnHostileAttackDrone(2)
	  DangerBase:SpawnHostileAttackFemale(1)
	  DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(60000)
   end
   if colonist:Random(100) < 5 and  UICity.day > 30 then
      DangerBase:StartRandDevil()
   end
end

function OnMsg.HostileOnDead(target)
	DangerBase.incident_data.hostiledead = DangerBase.incident_data.hostiledead + 1
	AddCustomOnScreenNotification("dangerAttackhostiledead", 
		T{"恭喜"}, T{"您摧毁一架魔王的:<target>",target=target.display_name}, "UI/Icons/Notifications/rocket.tga", 
		function(target)
		if IsValid(target) then ViewObjectMars(target:GetLogicalPos()) end
		end,
		{
			expiration = 15000,
			priority = "Important",
		})
end

function OnMsg.NewDay(day)
	SickOnDay(day)
   if UICity.day > 121 then return end
   -- DangerBase:SpawnHostileAttackDrone(10)
   -- DangerBase:StartHostileAttackRoverAttackDelay(30000)
   -- DangerBase:DronesRogue(10)
   -- DangerBase:SpawnHostileAttackRover(2)
   -- DangerBase:StartHostileAttackRoverAttackDelay(30000)
   
   local rand = UICity:Random(100)
   if rand < 5 and  UICity.day > 20 then
      DangerBase:StartRandDevil()
   end
    if rand < 20 and  UICity.day > 3
     then
       DangerBase:SpawnHostileAttackRover(UICity:Random(3))
	   DangerBase:SpawnHostileAttackDrone(5)
	   DangerBase:SpawnHostileAttackFemale(1)
       DangerBase:StartHostileAttackRoverAttackDelay(30000)
   end
   if day == 10 then
      DangerBase:StartRandDevil()
      DangerBase:DronesRogue(1)
	  DangerBase:SpawnHostileAttackFemale(1)
      DangerBase:DronesNoramlDelay(60000)
    elseif day == 20 then
      DangerBase:StartRandDevil()
      DangerBase:SpawnHostileAttackRover(2)
      DangerBase:DronesRogue(8)
	  DangerBase:SpawnHostileAttackDrone(2)
	  DangerBase:SpawnHostileAttackFemale(2)
      DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(60000)
   elseif day == 30 then
      DangerBase:StartRandDevil()
      DangerBase:SpawnHostileAttackRover(4)
      DangerBase:DronesRogue(8)
	  DangerBase:SpawnHostileAttackDrone(4)
	  DangerBase:SpawnHostileAttackFemale(4)
      DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(160000)
   elseif day == 40 then
      DangerBase:StartRandDevil()
      DangerBase:SpawnHostileAttackRover(8)
	  DangerBase:SpawnHostileAttackDrone(10)
	  DangerBase:SpawnHostileAttackFemale(4)
      DangerBase:DronesRogue(16)
      DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(160000)
   elseif day == 50 then
      DangerBase:StartRandDevil()
	  DangerBase:SpawnHostileAttackDrone(30)
      DangerBase:SpawnHostileAttackRover(15)
	  DangerBase:SpawnHostileAttackFemale(4)
      DangerBase:DronesRogue(32)
      DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(220000)
   elseif day == 60 then
      DangerBase:StartRandDevil()
      DangerBase:SpawnHostileAttackRover(10)
	  DangerBase:SpawnHostileAttackDrone(30)
	  DangerBase:SpawnHostileAttackFemale(4)
      DangerBase:DronesRogue(32)
      DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(440000)
   elseif day == 70 then
      DangerBase:StartRandDevil()
      DangerBase:SpawnHostileAttackRover(15)
	  DangerBase:SpawnHostileAttackDrone(30)
	  DangerBase:SpawnHostileAttackFemale(4)
      DangerBase:DronesRogue(32)
      DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(440000)
   elseif day == 80 then
      DangerBase:StartRandDevil()
      DangerBase:SpawnHostileAttackRover(10)
	  DangerBase:SpawnHostileAttackDrone(30)
	  DangerBase:SpawnHostileAttackFemale(4)
      DangerBase:DronesRogue(60)
      DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(440000)
   elseif day == 120 then
      DangerBase:StartRandDevil()
      DangerBase:SpawnHostileAttackRover(30)
	  DangerBase:SpawnHostileAttackDrone(60)
	  DangerBase:SpawnHostileAttackFemale(20)
      DangerBase:DronesRogue(100)
      DangerBase:StartHostileAttackRoverAttackDelay(30000)
      DangerBase:DronesNoramlDelay(640000)
   elseif day > 122 then
      MyOverNotification()
   end
end

function UnitDirectionModeDialog:UpdateInteractionObj(obj, pos)
  local stockpiles = obj and obj:GetAttaches("ResourceStockpileBase") or false
  local other_obj = not IsKindOfClasses(obj, "SharedStorageBaseVisualOnly", "StorageDepot") and stockpiles and not FindNearestObject(stockpiles, pos) and false
  local interaction_obj = false
  local block_goto
  if obj then
    interaction_obj, self.interaction_hint, block_goto = CityUnitController[UICity]:CanInteractWithObject(obj, self.interaction_mode)
    interaction_obj = interaction_obj and obj
  else
    interaction_obj = false
    self.interaction_hint = false
  end
  if not interaction_obj and other_obj and other_obj ~= obj then
    interaction_obj, self.interaction_hint, block_goto = CityUnitController[UICity]:CanInteractWithObject(other_obj, self.interaction_mode)
    interaction_obj = interaction_obj and other_obj
  end
  self.interaction_obj = interaction_obj
  self.block_goto = block_goto or GetDomeAtPoint(pos)
  if self.interaction_mode == "route" then
    self:UpdateTransportRouteVisuals()
  end
end

