
function MyGetObjectsByLabel(label)
  if label == "Working-age" then
    local ret = {}
    for _, dome in ipairs(UICity.labels.Dome or empty_table) do
      table.append(ret, dome.labels.Youth)
      table.append(ret, dome.labels.Adult)
      table.append(ret, dome.labels["Middle Aged"])
    end
    return ret
  else
    return UICity.labels[label]
  end
end

function AttackRover:GameInit()
  local city = self.city or UICity
  if self.reclaimed then
      city:RemoveFromLabel("HostileAttackRovers", self)
      -- city:AddToLabel("Rover", self)
      self.name = T{"超级战车"}
      self:SetCommand("Spawn")
  else 
	  city:RemoveFromLabel("Rover", self)
	  city:AddToLabel("HostileAttackRovers", self)
	  self.name = self.display_name
	  self:SetCommand("Spawn")
  end
end

function AttackRover:Attack(num_attacks, attacker)
  local target
  if not attacker then
    if num_attacks and num_attacks ~= 0 then
      self.attacks_remaining = Max(0, num_attacks) - 1
    else
      self.attacks_remaining = nil
      self:SetCommand("Roam")
    end
  end
  self.attack_look_for_target = true
  while true do
    target = self:PickTarget()
    if not target then
      if not self.reclaimed then
        self:RoamTick()
      else
        Sleep(1000)
        self:SetCommand("Idle")
      end
    else
      break
    end
  end
  self.attack_look_for_target = nil
  self:MoveInRange(target)
  local is_building = IsKindOf(target, "Building")
  if not IsValid(target) or is_building and target.destroyed or not is_building and target:IsDead() then
    Sleep(1000)
    return
  end
  -- 这里应该设置循环， 循环到达目的地
  if self:GetVisualDist2D(target) > self.attack_range then 
    return self:Idle()
  end
  self:FireRocket({target = target})
  self:SetCommand("Reload")
end

function AttackRover:MoveInRange(target)
  if self:GetVisualDist2D(target) <= self.attack_range then
    return
  end
  local is_building = IsKindOf(target, "Building")
  if is_building and target.destroyed or not is_building and target:IsDead() then
    return
  end
  self:Goto(target, self.attack_range, 0)
end

function AttackRover:CanInteractWithObject(obj, interaction_mode)
  if self.command == "Dead" then
    return false
  end
  if not self.reclaimed then
    return false, false, true
  end
  if interaction_mode == false or interaction_mode == "attack" then
    if IsKindOf(obj, "Building") or IsKindOf(obj, "BaseRover") or IsKindOf(obj, "Drone") then
	    if obj.command ~= "Dead" and obj.command ~= "Malfunction" then
		   return true
		end
		return true
    else
		return false
    end
  end
  if (self.interaction_mode == false or self.interaction_mode == "default" or self.interaction_mode == "move") and IsKindOf(obj, "Tunnel") and obj.working then
    return true
  end
  return false
end

function BaseRover:InteractWithObject(obj, interaction_mode)
  if not IsValid(obj) then
    return false
  end
  if self.command == "Dead" then
    return false
  end
  if interaction_mode == false or interaction_mode == "recharge" then
    if IsKindOf(obj, "ElectricityGridElement") and not IsKindOf(obj, "ConstructionSite") then
      self:SetCommand("RechargeFromGrid", obj)
      SetUnitControlInteractionMode(self, false)
    elseif obj ~= self and IsKindOf(obj, "BaseRover") and obj.command ~= "Dead" then
      self:SetCommand("EqualizePowerWithOtherRover", obj)
      SetUnitControlInteractionMode(self, false)
    end
  end
 
  if interaction_mode == false or interaction_mode == "attack" then
    if IsKindOf(self, "AttackRover") and (IsKindOf(obj, "Building") or IsKindOf(obj, "BaseRover") or IsKindOf(obj, "Drone")) then
		if obj.command ~= "Dead" and obj.command ~= "Malfunction" then
			-- self.last_attacker = obj
			-- self:SetCommand("Attack", -1, obj)
			SetUnitControlInteractionMode(self, false)
		end
    end
  end
  if (self.interaction_mode == false or self.interaction_mode == "default" or self.interaction_mode == "move") and IsKindOf(obj, "Tunnel") and obj.working then
    self:SetCommand("UseTunnel", obj)
    SetUnitControlInteractionMode(self, false)
  end
end

function AttackRover:ToggleAttackMode()
  if self.interaction_mode == "attack" then
    SetUnitControlInteractionMode(self, false)
  else
    SetUnitControlInteractionMode(self, "attack")
    SetUnitControlFocus(true, self)
  end
end

function AttackRover:ToggleAttackMode_Update(button)
  local to_mode = self.interaction_mode ~= "attack"
  button:SetIcon(to_mode and "UI/Icons/IPButtons/recharge.tga" or "UI/Icons/IPButtons/cancel.tga")
  button:SetRolloverTitle(T({40, "Recharge"}))
  button:SetRolloverText(T({
    41,
    "Select a power cable or another vehicle to recharge the battery."
  }))
  if not to_mode or not T({
    7509,
    "<left_click> Select target mode"
  }) then
  end
  button:SetRolloverHint((T({
    7510,
    "<left_click> on target to select it  <right_click> Cancel"
  })))
  if not to_mode or not T({
    7511,
    "<ButtonA> Select target mode"
  }) then
  end
  button:SetRolloverHintGamepad((T({
    7512,
    "<ButtonA> Cancel"
  })))
  button:SetEnabled(self:CanBeControlled())
end

function DefenceTower:DefenceTick()
  local mystery = self.city.mystery
  local hostile = self.city.labels.HostileAttackRovers or empty_table
  if IsValidThread(self.track_thread) then
    return
  end
  -- mod_print(#hostile)
  for i = 1, #hostile do
    local obj = hostile[i]
    if self:GetVisualDist2D(obj) <= self.shoot_range then
      self:OrientPlatform(obj:GetVisualPos(), 7200)
      if not IsValid(self) or not self.working or self.destroyed then
        return
      end
      self:FireRocket(nil, obj)
      Sleep(self.reload_time)
      self.is_firing = false
      return true
    end
  end
end

function Drone:GoRogue()
	if self.rogue or self:GetParent() then
		return
	end
	self.city:AddToLabel("RogueDrones", self)
	UpdateRogueNotification()
	self.rogue = true
	self:SetCarriedResource(false)
	if IsValid(self.command_center) then
		table.remove_entry(self.command_center.drones, self)
		self.command_center = nil
	end
	self:SetMoveAnim("rogueWalk")
	self:SetWaitAnim("rogueIdle")
	local sign = AttachToObject(self, "UnitSignRogue", "Origin")
	sign:SetVisible(true)
	sign:SetScale(const.SignsOverviewCameraScaleDown)
	self.MoveSleep = self.RogueMoveSleep
	RebuildInfopanel(self)
end

function Drone:RogueAttack(target)
  if not IsValid(target) then
    local target_label = self:Random(100) < self.rogue_attack_colonist_chance and "Colonist" or "Life-Support"
    local list = self.city.labels[target_label] or empty_table
    if #list == 0 then
      return self:RogueWander()
    end
    target = self.city:TableRand(list)
  end
  if not IsValid(target) then
    return
  end
  local is_unit = IsKindOf(target, "Colonist")
  local is_building = IsKindOf(target, "Building")
  self.rogue_intercepted = self.rogue_intercepted or {}
  self:StartMoving()
  self:PushDestructor(function(self)
    self.rogue_target = nil
  end)
  self.rogue_target = target
  self:GotoSameDomeAsObj(target)
  self:Goto(target, 2 * guim, 0)
  self:PopAndCallDestructor()
  if not IsValid(target) or is_unit and target:IsDying() or is_building and target.destroyed then
    return
  end
  if is_unit then
    if HexAxialDistance(self, target) > 1 then
      return
    end
    self:Face(target)
    self:SetAnim(1, "rogueAttack")
    target:SetCommand("AttackedByDrone", self)
    Sleep(self:TimeToAnimEnd())
    if IsValid(target.current_dome) then
      target.current_dome:AddPanicMarker(target:GetPos(), 15 * guim, 3 * const.HourDuration)
    end
  else
    local shape = GetEntityOutlineShape(target:GetEntity())
    local in_range = false
    local angle = target:GetAngle()
    local origin = target:GetPos()
    for i = 1, #shape do
      local q, r = shape[i]:xy()
      local x, y = HexToWorld(q, r)
      local dir = Rotate(point(x, y), angle)
      local pt = origin + dir
      if 2 > HexAxialDistance(self, pt) then
        in_range = true
        break
      end
    end
    if not in_range then
      return
    end
    PlayFX("DroneExplode", "start", self)
    DestroyBuildingImmediate(target)
    table.insert(g_AIDestroyedBuildings, target)
    self:SetCommand("Dead")
  end
end

function RocketProjectile:HitTarget(target)
  if IsKindOf(target, "Dome") then
  elseif IsKindOf(target, "Building") then
    PlayFX("MeteorDestruction", "start", target)
    -- DestroyBuildingImmediate(target)
    -- table.insert_unique(g_AIDestroyedBuildings, target)
	target:AccumulateMaintenancePoints(2000)
	if target:GetMaintenanceProgress() >= 100 or not target.accumulate_dust then
		DestroyBuildingImmediate(target)
		table.insert_unique(g_AIDestroyedBuildings, target)
	end
  elseif IsKindOf(target, "AttackRover") then
    PlayFX("MeteorDestruction", "start", target)
    target:RocketDamage(self.shooter)
  elseif IsKindOf(target, "DroneBase") then
    PlayFX("MeteorDestruction", "start", target)
    if not target:IsDead() then
      target:SetCommand("Dead")
    end
  end
end

function MyDronesRogue(number)
  local drones = MyGetObjectsByLabel("Drone")
  if number == -1 or number >= #drones then
    for _, drone in ipairs(drones) do
      drone:SetCommand("GoRogue")
    end
  else
    local affected = {}
    for i = 1, number do
      local r
      repeat
        r = 1 + UICity:Random(#drones)
      until not affected[r]
      affected[r] = true
    end
    for i, _ in pairs(affected) do
      drones[i]:SetCommand("GoRogue")
    end
  end
end

function myDronesNoraml()
 local drones = MyGetObjectsByLabel("Drone")
	for _, drone in ipairs(drones) do
		drone:SetCommand("RogueRevertToNormal")
	end
end
function MyAttack()
	local drones = MyGetObjectsByLabel("AttackRover")
	for _, drone in ipairs(drones) do
		if not drone.reclaimed then
		    drone:SetCommand("Reload")
			drone:SetCommand("Attack", -1)
		end
	end
end
function TestAttackRover(number)
    AddCustomOnScreenNotification("danger", 
		T{"警报"}, T{"<number> 个敌人即将到来",number=number}, "UI/Icons/Notifications/rocket.tga", 
		false,
		{
			expiration = 15000,
			priority = "Important",
		})	
    for i = 1, number do
		PlaceObject("AttackRover", {
			use_random_pos = true,
			can_repair = true,
			attack_range = 750 * guim,
			attack_target_labels = {
				"OutsideBuildingsTargets",
				"Rover",
				"Drone"
			}
		})
    end
    -- rover2:SetCommand("Attack", 3)
end

function RandDevil()
 local number = UICity:Random(4)
 if number == 0 then
     CheatDustDevil("great")
 elseif number == 1 then
     CheatDustDevil("electrostatic")
 elseif number == 2 then
     CheatColdWave()
 elseif number == 3 then
     CheatColdWave()
     CheatDustDevil("electrostatic")
 else
     CheatDustDevil("great")
     CheatColdWave()
     CheatDustDevil("electrostatic")
 end
end

function AddRenegadeNColonists(number)

  if number == -1 or number >= #UICity.labels.Colonist then
    for _, colonist in ipairs(UICity.labels.Colonist) do
      colonist:AddTrait("Renegade")
    end
  else
    local affected = {}
    for i = 1, number do
      local r
      repeat
        r = 1 + UICity:Random(#UICity.labels.Colonist)
      until not affected[r]
      affected[r] = true
    end
    for i, _ in pairs(affected) do
      UICity.labels.Colonist[i]:AddTrait("Renegade")
    end
  end
end

function OnMsg.ColonistBorn(colonist, event)
   local rand = colonist:Random(100)
   if event == "born" and rand < 20 and  UICity.day > 20
     then
      local dome_id = UICity:Random(#UICity.labels.Dome)
      StartBombard(UICity.labels.Dome[dome_id], 500*guim, 20, 2000, 6000)
   end
   if event == "born" and rand < 10 and  UICity.day > 20
     then
      AddRenegadeNColonists(5)
   end
   if event == "born" and rand < 5 and  UICity.day > 20
     then
      RandDevil()
   end
end

function OnMsg.ColonistArrived(colonist)
  if colonist:Random(100) < 10 and  UICity.day > 30
     then
      TestAttackRover(2)
      MyDronesRogue(2)
      DelayedCall(30000, MyAttack)
      DelayedCall(60000, myDronesNoraml)
   end
   if colonist:Random(100) < 5 and  UICity.day > 30
     then
      RandDevil()
   end
end

function MyHelpNotification()
	CreateRealTimeThread(WaitCustomPopupNotification,
		T{"开启危险模式"},
		T{"尊敬的指挥官，您已经开启了危险模式，因此10天后将会有源源不断的攻击到来. <newline>我赠送了您3辆火箭战车并且帮您开启了防御炮塔，请努力活过120天吧.<newline>祝您好运，哈哈",},
		{ T{"如果您120天后仍旧幸存，请告知我. 我是Alexliyu, 这个世界的魔王"} })
end

function MyOverNotification()
	
	CreateRealTimeThread(function()
	        local title = T{"恭喜"}
		local text = T{"您是一个真正的英雄, 这里已经没有任何事情能阻挡您的脚步，我将会离开，祝好运.",}
		local choice = {
			T{"继续"},
			T{"再见"},
		}
		local res = WaitCustomPopupNotification(title, text , choice)
		    if res == 2 then
			CreateRealTimeThread(function()
			  OpenPreGameMainMenu()
			end)
		    end
	end)
end

function MyStartDanger()
     GrantTech("DefenseTower")
     if not UICity.mystery then
       UICity.mystery = MysteryBase
    end

    UICity.mystery.can_shoot_rovers = true
    UICity.mystery.enable_rover_repair = true
    UICity.mystery.reclaim_repaired_rovers = true
    if not UICity.labels.AttackRover or #UICity.labels.AttackRover < 1 then
	    for i = 1, 3 do
	      local rover = PlaceObject("AttackRover", {
		      use_random_pos = true,
		      can_repair = true,
		      reclaimed = true,
		      attack_range = 500 * guim,
		      palettes = {"AttackRoverRed"}
		    })
		    
	       UICity:RemoveFromLabel("HostileAttackRovers", rover)
	       -- UICity:AddToLabel("Rover", rover)
	       CommandObject.SetCommand(rover, "Idle")
	    end
    end
   -- PlaceObj("TechEffect_UnlockResupplyItem", {"Item", "AttackRover"})
end

function OnMsg.LoadGame()
    MyHelpNotification()
    MyStartDanger()
end
function OnMsg.MysteryChosen()
    MyHelpNotification()
    MyStartDanger()
end


function OnMsg.NewDay(day)

   if UICity.day > 121 then
	return
   end
   TestAttackRover(2)
   DelayedCall(30000, MyAttack)
   
   local rand = UICity:Random(100)
   if rand < 5 and  UICity.day > 20
     then
      RandDevil()
   end
    if rand < 20 and  UICity.day > 3
     then
       TestAttackRover(UICity:Random(5))
       DelayedCall(30000, MyAttack)
   end
   if day == 10 then
      RandDevil()
      MyDronesRogue(1)
      DelayedCall(60000, myDronesNoraml)
    elseif day == 20 then
      RandDevil()
      TestAttackRover(2)
      MyDronesRogue(8)
      DelayedCall(30000, MyAttack)
      DelayedCall(60000, myDronesNoraml)
   elseif day == 30 then
      RandDevil()
      TestAttackRover(4)
      MyDronesRogue(8)
      DelayedCall(30000, MyAttack)
      DelayedCall(160000, myDronesNoraml)
   elseif day == 40 then
      RandDevil()
      TestAttackRover(16)
      MyDronesRogue(16)
      DelayedCall(30000, MyAttack)
      DelayedCall(160000, myDronesNoraml)
   elseif day == 50 then
      RandDevil()
      TestAttackRover(32)
      MyDronesRogue(32)
      DelayedCall(30000, MyAttack)
      DelayedCall(220000, myDronesNoraml)
   elseif day == 60 then
      RandDevil()
      TestAttackRover(32)
      MyDronesRogue(32)
      DelayedCall(30000, MyAttack)
      DelayedCall(440000, myDronesNoraml)
   elseif day == 70 then
      RandDevil()
      TestAttackRover(32)
      MyDronesRogue(32)
      DelayedCall(30000, MyAttack)
      DelayedCall(440000, myDronesNoraml)
   elseif day == 80 then
      RandDevil()
      TestAttackRover(40)
      MyDronesRogue(60)
      DelayedCall(30000, MyAttack)
      DelayedCall(440000, myDronesNoraml)
   elseif day == 120 then
      RandDevil()
      TestAttackRover(80)
      MyDronesRogue(100)
      DelayedCall(30000, MyAttack)
      DelayedCall(640000, myDronesNoraml)
   elseif day > 122 then
      MyOverNotification()
   end
end

function OnMsg.ClassesBuilt()
	AutoHelpAddInfoSection()
    PlaceObj("Cargo", {
		description = T({
		5309,
		"Recycles up to 70% of the Water used in the Dome."
		}),
		group = "Rovers",
		id = "AttackRover",
		name = T{"超级战车"},
		price = 500000000,
		kg = 20000,
	})
end


function AutoHelpAddInfoSection()
    table.insert(XTemplates.ipAttackRover[1], 
        PlaceObj("XTemplateTemplate", {
			"comment",
			"move",
			"__condition",
			function(parent, context)
				return context.reclaimed and not context:CanBeRepaired()
			end,
			"__template",
			"InfopanelButton",
            "Icon", "UI/Icons/Upgrades/factory_ai_02.tga",
            "RolloverTitle", T{"自动救援"},
            "OnPressParam",
            "ToggleAttackMode"
        }, {
            PlaceObj("XTemplateFunc", {
                "name", "OnActivate(self, context)", 
                "parent", function(parent, context)
                        return parent.parent
                    end,
                "func", function(self, context)
                        context.auto_help = not context.auto_help
                        ObjModified(context)
                    end
            })
        })
    )
end